/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert                                  *
 *   roweigert@t-online.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 * @file databaseinterface.h
 *
 * This file holds the class for the generic parts of databaseinterface.<br>
 * The databasedrivers, are chosen and handled by this class.<br>
 * @author Roland Weigert
 * @date 26-09-2008
 * @todo Write all
 */

#ifndef _DATABASEINTERFACE_H_
#define _DATABASEINTERFACE_H_

#include <QString>
#include <QStringList>
#include <QObject>
#include <QImage>
#include <QCompleter>

#include "global-stuff/globals.h"

/**
 * Use this struct for interchange with the databaseinterface-class<br>
 * This makes it independent from all other classes.
 */
struct dbtrackdescription
  {
    int tracknumber;
    QString trackname;
    QString artist;
    QString album;
    bool albumva;
    QString genre;
    QString release;
    QImage frontcover;
    QImage mediacover;
    QImage backcover;
  };

/**
 * @brief databaseinterface
 * This is the generalised interface to all databseinterfaces.<br>
 * It is directly controlled by databsethread, that only passes through all calls for the database.
 * All db-core-interfaces are directly direved from this purely abstract class.
 */
class databaseinterface : public QObject
  {
    Q_OBJECT
  public:
	  databaseinterface(QString &dbname, QString &user, QString &pass);
    ~databaseinterface();
  public slots:
    virtual void checkdbinsert(const dbtrackdescription curtrack){};
    /**
     * @brief getdbcover
     * Catch a coverfile from db. The id is from a prior dblookup.<br>
     * Connect to the gotdbcover signal to get the picture back from the db.<br>
     * If something goes wrong, or a non existing id is given the signal dbcovererror is emitted.
     * @param id The id of the cover
    */
    virtual void getdbcover(const long id){};
    /**
     * @brief getdbbytracknumberslot
     * Find out the albumid of the current track given by the information in curtrack.
     * Than fetch the Track from the db that has right albumid and tracknumber;
     * Connect to gotdbartistandtrack-signal to get the result.
     * @param tracknumber The number of the searched track
     * @param track The dbtrackdescription of the current track
     */
    virtual void getdbbytracknumber(const int tracknumber,const dbtrackdescription track){};
    virtual void getdbbytrackname (const QString trackname, const dbtrackdescription track){};
    virtual void getdbbyartistname(const QString artistname, const dbtrackdescription track){};
    virtual void getdbbyalbumname(const QString albumname, const dbtrackdescription track){};
    virtual void filldbartists(QCompleter *artists){};
    virtual void fillbalbums(QCompleter *albums){};
    virtual void filldbtracks(QCompleter *tracks){};
    virtual void filldbgenres(QCompleter *genres){};
    /**
     * @brief checkdbfileinsert
     * Checks if a file is already known in the Files table.<br>
     * If The file is not yet in the table gotdbfileisnew-signal is emitted.<br>
     * You can call dbfileinsert than with this filename, and the read out tags.<br>
     * If the file is already known, but the modificationtime is newer than in the db the signal gotdbfileneedsupdate is emitted.<br>
     * You can call dbfileupdate than.<br>
     * @param filename The filename to check
     * @param modified The Modificationdate of the file in the Format YYYY-MM-DD HH:MM:SS
     */
    virtual void checkdbfileinsert( const QString filename,const QString modified){};
    /**
     * @brief dbfileinsert
     * Inserts a file into the database that is not yet in it.<br>
     * The QStringList insertslist is checked, before inserting, so it makes no sense to call this slot without checking if the file is in the db with checkdbfileinsert before.<br>
     * @param filename The filename to insert
     * @param tags The tags of the file
     */
    virtual void dbfileinsert(const QString filename,dbtrackdescription tags){};
    /**
     * @brief dbfileupdate
     * Updates a file in the database that has a newer timestamp than in db.<br>
     * The QStringList updatelist is checked, before updating, so it makes no sense to call this slot without checking if the file is in the db with checkdbfileinsert before.<br>
     * @param filename The filename to update
     * @param tags The tags of the file
     */
    virtual void dbfileupdate(const QString filename,dbtrackdescription tags){};
    /**
     * @brief Check if the db is already started and connected.
     * You can start a db with startdb, if it is not already running.
     */
    virtual bool isdbconnected(){};
  signals:
    void gotdbcover(const QImage);
    void gotdbartistandtrack( const QString Artist, const QString Track);
    void gotdbfileisnew(const QString);
    void gotdbfileneedsupdate(const QString);
    void dbcovererror(long);
  private:
    /**
    * @brief Start a not running db
    * Give username and pass to start the db.
    * The db will be generated with the given userdata, if it is not existing already.
    * The user is asked for a db-pass with create-abilities therefor.
     */
    virtual void startdb(QString &dbname, QString &user, QString &pass){};
    QStringList insertlist;
    QStringList updatelist;

  };

#endif
