/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file freedbinterface.h
 *The class that allows connecting to freedbservers.<br>
 *Contains highlevel-functions for fetching and uploading of discs.<br>
 *It also has a function that queries for a serverlist.<br>
 *Since Freedb isn't maintained anymore, there is only one server atm, but maybe this changes in the future.<br>
 *
 *There are also some helperstructs inside this header, that are needed for interaction, withe the class.<br>
 *Finally there is an enum with querytypes,<br>
 *
 *cdinterface.h must be included, cause it needs the toc-information.<br>
 *Following needs to defined in a globals.h-file when you want to use this class.<br>
 *USERNAME, APPNAME , VERSION
 *
 *@author Roland Weigert
 *@date 29-07-2007
 *@todo Write upload-functions
 */

#ifndef _FREEDBINTERFACE_H_
#define _FREEDBINTERFACE_H_

#include <QString>
#include <QStringList>
#include <QList>
#include <QHttp>
#include <QObject>
#include <QByteArray>

#include "globals.h"
#include "cdinterface.h"

//Prototypes
struct freedbtrack;
struct freedbalbum;
struct freedbdiscquery;


/**
*@brief enum for the various querytypes.
*This enum is used, when the querytype for the memberfunction sendquery() needs to be set.
*/
enum querytype
{
  /**fetch a discinformation, for a discid*/
  get=1,
    /**Upload a discinformation to freedb*/
    send=2,
    /**Send query for a serverlist*/
    serevers=4
  };

/**
*@brief The class for all interactions with freedb
*/
class freedbinterface : public QObject
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *
    *@param server The name of the server to contact
    */
    freedbinterface( QString server);
    /**
    *@brief Destructor
    */
    ~freedbinterface();
    /**
    *@brief Getservers-function
    *
    *Start a query for freedb-serverlist.<br>
    *Connect to Signal querryerror(const QString) to see if this query failed.<br>
    *Connect to gotserverlist (const QStringlist) to receive the serverlist.<br>
    */
    void getservers();
    /**
    *@brief Getdisc-function
    *
    *Start a query for the given disk.<br>
    *Connect to Signal querryerror(const QString) to see if this query failed.<br>
    *Connect to gotdisc (const QList <freedbalbum>) to receive the album(s).<br>
    *@param discinformation The cdinterface that holds a valid tocinformation
    *
    */
    void getdisc(cdinterface &discinformation,const QString freedb_discid);
  private:
    void sendquery (const QString querytext, querytype type);
    const QStringList chopresult(const QByteArray &);
    QString servername;
    QHttp *connection;
    toc *disc;
    int tracks;
    QString discid;
    QStringList bufferlist;
    QList<freedbdiscquery> genrelist;
    QList<freedbdiscquery>::Iterator genrewalkingpointer;
    QList<freedbalbum> albumresult;
  public slots:
    /**Call this slot, if working queries need to be aborted*/
    void abortfetch();
  private slots:
    void parseservers(bool);
    void parsediscs(bool);
    void parsedisc(bool);
  signals:
    /**This Signal is emitted, when freedb answers with an error.*/
    void queryerror(const QString errortext);
    /**This Signal is emitted, when the reult,of an discquery is sent by freedb*/
    void gotdisc (const QList<freedbalbum> albumresult);
    /**This Signal is emitted, when the serverlist was sent by freedb*/
    void gotserverlist(QStringList);
    /**This Signal is emitted, when the upload of a disk to freedb is done*/
    void uploadcomplete();
  };

struct freedbtrack
  {
    int fdb_number;
    QString fdb_title;
  };

struct freedbalbum
  {
    QString fdb_album;
    QString fdb_artist;
    QString fdb_genre;
    QString fdb_discid;
    int fdb_release;
    QList<freedbtrack> fdb_tracklist;
  };

struct freedbdiscquery
  {
    QString discid;
    QString genre;
  };

#endif
