/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file fileoperator.h
 *
 *This file holds the class for fileoperations<br>
 *It handles all fileoperations.<br>
 *It is needed, because several classes do fileoperations, and it is easier to<br>
 *delete empty directories, when only one class does all deleting, and moving.<br>
 *Also the maintenance gets easier, because there is no extra code in the other classes<br>
 *needed anymore. Only one handle for the whole programm
 *@author Roland Weigert
 *@date 21-02-2009
 *@todo Write pictures in destinationdirectories
 */

#ifndef _FILEOPERATOR_H_
#define _FILEOPERATOR_H_

#include <QDir>
#include <QFile>
#include <QList>
#include <QStringList>
#include <QString>
#include <QMessageBox>
#include <QObject>
#include <QMutex>
#include <QThread>

#include "globals.h"
#include "mediafile.h"
#include "settings/rcfile.h"

#include "global-stuff/communicationinterface.h"


struct fileentry;

class fileoperator : public QThread
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *Initialises all lists
    *@param deletemode sets the deletemode for this object. You can also set it with setdeletemode()
    *@param ovmode sets the behaviour, when a file is already present
    *@param ignorelist Set the list of files, that are not counted,when directory is checked for emptyness
    *@param basedirectory the basedir all copied files get moved to when operations based on tags are used.
    *@param nodeletelist the list with diectories recursivemode should not delete (tempdir,workingdir,...)
    */
    fileoperator(const int deletemode,const int ovmode,const QStringList ignorelist, const QString basedirectory,const QStringList nodeletelist);
    /**
    *@brief Destructor
    */
    ~fileoperator();
    /**
    *@brief change the deletemode
    *@param mode The new deletemode
    */
    void setdeletemode(const int mode);
    /**
    *@brief Set a new ignorelist
    *@param list a list with new files for the ignorelist
    */
    void setignorelist(const QStringList list);
    /**
    *@brief Set a new nodeletelist
    *@param list a list with directories, that should not be deleted in recursivemode
    */
    void setnodeletelist(const QStringList list);
    /**
    *@brief Set the overwritemode
    *@param mode The new overwritemode.
    */
    void setovmode(const int mode);
    /**
    *@brief Set the basedir, where the path starts, when files are moved or copied by tags
    */
    void setbasedir(const QString);
    /**
    *@brief Sets the filter for generating of filenames and directories
    *@param filter The filter to use from now on
    */
    bool setfilter(const QString);
    /**
    *@brief Add a new file that is to delete on the joblist
    *@param filename The file that is to delete*/
    void deletefile(const QString filename);
    /**
    *@brief Add a new file that is to copy on the joblist
    *@param source The sourcefilename
    *@param destination The destinationfilename
    */
    void copyfile(const QString source, const QString destination);
    /**
    *@brief Add a new file that is to move on the joblist
    *@param source The sourcefilename
    *@param destination The destinationfilename
    */
    void movefile(const QString source, const QString destination);
    /**
    *@brief Add a new file that is to copy on the joblist
    *@param source The sourcefilename
    *The destinationame is generated out of the tags
    */
    void copyfile(const QString source);
    /**
    *@brief Add a new file that is to move on the joblist
    *@param source The sourcefilename
    *The destinationame is generated out of the tags
    */
    void movefile(const QString source);
  signals:
    /**
    *@brief This signal is emitted, when a file is ready processed
    */
    void filedone(const QString filename);
    /**
    *@brief This signal is emitted, when an operation could not be done, because of missing tags
    */
    void nottagged(const QString filename);
    /**
    *@brief This signal is emitted, when the destinationfilename is not writable
    */
    void nowrite(const QString sourcefile, const QString destinationfile);
    /**
    *@brief This signal is emitted, when the file could not be deleted
    */
    void nodelete(const QString filename);
    /**
    *@brief This signal should inform the user, that no valid filter is set yet, so no tagbased moving can be used
    *Restart with restartslot, and the filname got by this signal, to retry new filter.
    @param filename The actual processed file
    */
    void nofilter(const QString filename);
    void fileready(QString);
  public slots:
    /**
    *@brief Retries operation with file
    *@param filename This filename is searched in the joblist, so only existing jobs really restart
    */
    void retry(const QString filename);
    /**
    *@brief Resumes operation after file
    *@param filename This filename is searched in the joblist, so only existing jobs really restart
    */
    void resume(const QString filename);
  private slots:
    void nextjob(const QString readyfile);
    void nottaggedslot(const QString filename);
    void nowriteslot(const QString sourcefile, const QString destinationfile);
    void nodeleteslot(const QString filename);
  private:
    QMutex locklist;
    int deletemode;
    int ovmode;
    QStringList ignorelist;
    bool taskrunning;
    QList<fileentry> joblist;
    /**The following variables are needed for moving with tagsupport*/
    QString filterstring;
    QStringList filterparts;
    QString basedir;
    int partcount;
    bool filterworks;
    QStringList nodeletelist;
    fileentry *actualjob;
    void processjob(fileentry &);
    void startjob();
    const QString generatenameandpath(const QString filename);
    /**
    *@brief Checks for ovmode, and processes file
    *This is only convenience, to have the code only once.
    */
    bool ovcheck(const QString source, const QString destination);
    /**
    *@brief Deletes given directories recursively.
    *It also removes all files, that are on the ignorelist.<br>
    */
    void deletedir(const QString);
    void deleterecursivedir(const QString);
    QString replaceunsuported(const QString);
  protected:
    void run();
  };

/**
*@brief This struct is used for the joblist
*/
struct fileentry
  {
    /**The sourcefile*/
    QString source;
    /**The destinationfilename, if known, (deletes dont have a destination)*/
    QString destination;
    /**The type of this job*/
    int jobtype;
    /**The status of this job*/
    int status;
  };

#endif
