/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file cdparanoiainterface.h
 *
 *This file holds the class for communication with cdparanoia<br>
 *You can send as many requests, as you want, they are all executed one by one, till the list is done.<br>
 *@author Roland Weigert
 *@date 30-08-2007
 */

#ifndef _CDPARANOIAINTERFACE_H_
#define _CDPARANOIAINTERFACE_H_

#include <QString>
#include <QList>
#include <QProcess>
#include <QObject>
#include <QFileInfo>
#include <QDir>

#include "global-stuff/globals.h"
#include "global-stuff/fileoperator.h"
#include "global-stuff/communicationinterface.h"

//Prototypes
struct cdparanoiaprocess;

class cdparanoiainterface : public QObject
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    */
    cdparanoiainterface();
    /**
    *@brief Destuctor
    */
    ~cdparanoiainterface();
    /**
    *@brief Checks if cdparanoia is installed
    *@return TRUE if cdparanoia is found, FALSE if not.
    */
    const bool paranoiaok();
    /**
    *@brief Put a new track on the toreadlist
    *@param tracknumber The tracknumber to read
    *@param path The path, the files go to
    */
    void readtrack(const int tracknumber,const QString path);
    const bool isrunning() const;
  public slots:
    void cancelread(int track);
    void cancelall();
    void requestslot(QStringList destination,messageobject *message);
    void requestslot(QString sender,QString type, QVariant data);
    void requestslot(QString sender,QString type, QList<QVariant> data);
  signals:
    /**
    *@brief Connect to this slot, to get informed, when track is ready read
    *@param savename The filename the track was saved to
    *@param tracknumber The tracknumber of the read track
    */
    void trackreadready(const QString savename, const int tracknumber);
    /**
    *@brief connect to this slot, to get infomed, when cdparanoia fails
    *@param errortext The errortext
    */
    void paranoiaerror(const QString errortext);
    /**
    *@brief Connect to this signal to get the status of ripping
    *@param value The process in percent
    *@param tracknumber The tracknumber, that is read atm
    */
    void progresssignal(const int value,int tracknumber);
  private slots:
    void processtracking();
    void processfinished(int);
  private:
    QList<cdparanoiaprocess> processlist;
    bool taskrunning;
    QFile deletefile;
    bool abortrunning;
  };

/**
*@brief This struct is used by cdparanoiainterface, to keep a list of processes, that have to be started
*/
struct cdparanoiaprocess
  {
    /**The startsector, is set, when process gets its first results from cdparanoia*/
    long start;
    /**The endsector, is set, when process gets its first results from cdparanoia*/
    long end;
    /**Process running or not*/
    bool running;
    /**Process ready or not*/
    bool ready;
    /**Tracknumber of process*/
    int tracknumber;
    /**The savename of the track*/
    QString filename;
    /**The process for this track*/
    QProcess *readprocess;
  };
#endif
