/***************************************************************************
 *   Copyright (C) 2007 by Roland Weigert   *
 *   roweigert@t-online.de             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 *@file cdparanoiainterface.h
 *
 *This file holds the class for communication with lame<br>
 *You can send as many requests, as you want, they are all executed one by one, till the list is done.<br>
 *You can tell in constructor how many processes are allowed simultinously.<br>
 *All progresssignals sen negative tracknumbers, so the progresswidget doesnt mix the processes with<br>
 *cdparanoiainterface.
 *@author Roland Weigert
 *@date 05-01-2009
 */

#ifndef _LAMEINTERFACE_H_
#define _LAMEINTERFACE_H_

#include <QString>
#include <QList>
#include <QProcess>
#include <QObject>
#include <QFileInfo>
#include <QDir>
#include <QVariant>

#include "global-stuff/globals.h"
#include "global-stuff/fileoperator.h"
#include "global-stuff/communicationinterface.h"
//Prototypes
class lameprocess;

class lameinterface :public QObject
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *@param encodercount The number of encoders allowed to run at the same time
    */
    lameinterface(int encodercount);
    /**
    *@brief Destructor
    */
    ~lameinterface();
    /**
    *@brief Check if lame is installed
    *@return TRUE if lame is found
    */
    bool lameok();
    /**
    *@brief Put track on the encodinglist. It is started, when there is a free slot for it
    *@param track The  tracknumber, used for the progresssignals, of the encoded file
    *@param path The path where the sourcefile is in, and the encoded file goes in (named track##.mp3)
    *@param bitrate The bitrate taht should be used (0=VBR)
    *@param stereomode use audiomode::msjs defined in globals.h
    *@param high Use Highflag with lame
    *@param fast Use fast encoding with lame
    */
    void encodetrack(const int track,const QString path,const int bitrate,const int stereomode,bool high, bool fast);
    const bool isrunning();
  public slots:
    void cancelencode(int track);
    void cancelall();
    void requestslot(QStringList destination,messageobject *message);
    void requestslot(QString sender,QString type, QVariant data);
    void requestslot(QString sender,QString type, QList<QVariant> data);
  signals:
    /**
    *@brief Connect to this slot, to get informed, when track is ready encoded
    *@param savename The filename the track was saved to
    *@param tracknumber The tracknumber of the track to encode
    */
    void trackencodeready(const QString savename, const int tracknumber);
    /**
    *@brief connect to this slot, to get infomed, when lame fails
    *@param errortext The errortext
    */
    void lameerror(const QString errortext);
    /**
    *@brief Connect to this signal to get the status of encoding
    *@param value The process in percent
    *@param tracknumber The tracknumber, that is encoded atm
    */
    void progresssignal(const int value,int tracknumber);
    void allencoded();
  private:
    QList<lameprocess *> encodelist;
    int runningtasks;
    int numcoders;
  private slots:
    void gottrackencodeready(const QString savename, const int tracknumber);
  };


/**
 *@brief The lameprocess-class
 *We need to write an own class here, so we can have more than one process running, without getting confused by the<br>
 *readyread-signal of Qt. In that way we can catch all info by all the running tasks without the difficulty of judging<br>
 *where the data belongs to.
 */
class lameprocess : public QObject
  {
    Q_OBJECT
  public:
    /**
    *@brief Constructor
    *@param track The  tracknumber, used for the progresssignals, of the encoded file
    *@param path The path where the sourcefile is in, and the encoded file goes in (named track##.mp3)
    *@param bitrate The bitrate taht should be used (0=VBR)
    *@param stereomode use audiomode::msjs defined in globals.h
    *@param high Use Highflag with lame
    *@param fast Use fast encoding with lame
    */
    lameprocess(const int track,const QString path,const int bitrate,const int stereomode,bool high, bool fast);
    /**
    *@brief Destructor
    */
    ~lameprocess();
    /**
    *@brief Start the initialized process
    */
    void start();
    /**
    *@brief Check if this process is running,or not.
    *@return 0 if not yet started, 1 if running, 2 if it is finished
    */
    const int isrunning() const;
    /**
    *@brief Returns the tracknumber used for this process
    */
    const int gettracknumber() const;
    /**
    *@brief Returns the Path of the encoded file
    */
    const QString getpath() const;
  public slots:
    void cancelencode();
  signals:
    /**
    *@brief Connect to this slot, to get informed, when track is ready encoded
    *@param savename The filename the track was saved to
    *@param tracknumber The tracknumber of the read track
    */
    void trackencodeready(const QString savename, const int tracknumber);
    /**
    *@brief connect to this slot, to get infomed, when lame fails
    *@param errortext The errortext
    */
    void lameerror(const QString errortext);
    /**
    *@brief Connect to this signal to get the status of encoding
    *@param value The process in percent
    *@param tracknumber The tracknumber, that is encoded atm
    */
    void progresssignal(const int value,int tracknumber);
  private slots:
    void processtracking();
    void processfinished(int);
  private:
    int tracknumber;
    QString source;
    QString destination;
    QProcess *process;
    QString executable;
    QStringList parameters;
    int taskrunning;
    QFile deletefile;
  };

#endif
